package com.progress.print;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.print.attribute.HashPrintRequestAttributeSet;
import javax.print.attribute.HashPrintServiceAttributeSet;
import javax.print.attribute.PrintRequestAttributeSet;
import javax.print.attribute.PrintServiceAttributeSet;
import javax.print.attribute.standard.Copies;
import javax.print.attribute.standard.PrinterName;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRExporterParameter;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.data.JRXmlDataSource;
import net.sf.jasperreports.engine.export.JRHtmlExporter;
import net.sf.jasperreports.engine.export.JRPdfExporter;
import net.sf.jasperreports.engine.export.JRPrintServiceExporter;
import net.sf.jasperreports.engine.export.JRPrintServiceExporterParameter;
import net.sf.jasperreports.engine.export.JRRtfExporter;
import net.sf.jasperreports.engine.export.JRTextExporter;
import net.sf.jasperreports.engine.export.JRTextExporterParameter;
import net.sf.jasperreports.engine.export.JRXlsExporter;
import net.sf.jasperreports.engine.export.JRXlsExporterParameter;
import net.sf.jasperreports.engine.export.JRXmlExporter;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.PosixParser;
import org.apache.log4j.Logger;

public class JasperPrint implements Constants {
    private static final Logger LOGGER = Logger.getLogger(JasperPrint.class);

    private static Options options = null; // Command line options

    private CommandLine cmd = null; // Command Line arguments

    private String outFileName = null;

    private String outFileFormat = DEFAULT_FORMAT;

    private String inDataFile = null;

    private String reportFileName = null;

    private boolean doCompile = false;

    private boolean doDelete = false;

    private boolean doDeleteXml = false;

    private boolean doSaveSeparatly = false;

    private String printerName = null;

    private String emails = null;

    private String subject = null;

    private String body = null;

    private static List fileNames = new ArrayList();

    private int numberOfCopies = 1;

    static {
	options = new Options();
	options.addOption(TEMPLATE_LOCATION_OPTION, true,
		"Template file location");
	options.addOption(COMPILE_OPTION, false, "Compile option");
	options.addOption(PRINTER_OPTION, true, "Printer location option");
	options.addOption(DATA_LOCATION_OPTION, true, "Data file location");
	options.addOption(FORMAT_OPTION, true, "Format option. \n"
		+ "Possible values: pdf, xls, xml, txt, rtf, print. "
		+ DEFAULT_FORMAT + " by default ");
	options.addOption(OUTPUT_FILE_OPTION, true, "Output file.");
	options.addOption(DELETE_AFTER_GENERATION, false,
		"Delete generated file.");
	options.addOption(DELETE_DATA_AFTER_GENERATION, false,
		"Delete data source file after generation.");
	options.addOption(NUMBER_OF_COPIES, true, "Number of copies to print.");

	options.addOption(EMAIL_OPTION, true,
		"Email address[es], delimiter ','.");
	options.addOption(SUBJECT_OPTION, true, "Email subject.");
	options.addOption(BODY_OPTION, true, "Email body text.");
	options.addOption(SAVE_EACH_PAGE_ON_FILE, false,
		"Save each page separatly.");
    }

    public static void main(String[] args) throws Exception {
	JasperPrint printUtil = new JasperPrint();
	printUtil.loadArgs(args);
	printUtil.operate();
    }

    private void operate() throws Exception {
	List printList = new ArrayList();
	JRXmlDataSource jrxmlds = null;
	HashMap hm = new HashMap();

	if (getInDataFile() != null) {
	    String recordPath = "";
	    XmlParser xp = new XmlParser();
	    xp.setFileName(getInDataFile());
	    xp.parse();
	    String rootElName = xp.getRootName();
	    if (rootElName == null) {
		LOGGER.error("Error parsing xml file: " + getInDataFile());
		System.exit(1);
	    } else {
		recordPath = "/" + rootElName + "/row";
	    }
	    hm = xp.getParameters();
	    if ("print".equals(getOutFileFormat().toLowerCase()) && getNumberOfCopies() > 1) {
		for (int i = 1; i <= getNumberOfCopies(); i++) {
		    jrxmlds = new JRXmlDataSource(getInDataFile(), recordPath);
		    hm.put(NUMBER_OF_COPY, new Integer(i));
		    printList.add(JasperFillManager.fillReport(
			    getReportFileName(), hm, jrxmlds));
		}
	    } else {
		jrxmlds = new JRXmlDataSource(getInDataFile(), recordPath);
	    }
	}

	try {
	    if (isDoCompile()
		    || "jrxml".equals(getFileExtension(getReportFileName())
			    .toLowerCase())) {
		setReportFileName(JasperCompileManager
			.compileReportToFile(getReportFileName()));
		if (isDoCompile()) {
		    System.exit(1);
		}
	    }

	    net.sf.jasperreports.engine.JasperPrint print  = null;
	    if (!("print".equals(getOutFileFormat().toLowerCase()) && getNumberOfCopies() > 1)) {
		print = JasperFillManager.fillReport(getReportFileName(), hm,
			jrxmlds);
	    }

	    if (!getOutFileFormat().toLowerCase().equals(
		    getFileExtension(getOutFileName()).toLowerCase())
		    && (getFileExtension(getOutFileName()).toLowerCase()
			    .equals("html")
			    || getFileExtension(getOutFileName()).toLowerCase()
				    .equals("txt")
			    || getFileExtension(getOutFileName()).toLowerCase()
				    .equals("rtf")
			    || getFileExtension(getOutFileName()).toLowerCase()
				    .equals("xml") || getFileExtension(
			    getOutFileName()).toLowerCase().equals("xls"))) {
		setOutFileFormat(getFileExtension(getOutFileName())
			.toLowerCase());
	    }

	    if (isDoSaveSeparatly()
		    && !"print".equals(getOutFileFormat().toLowerCase())) {

		List fileNames = new ArrayList();
		if (print.getPages().size() > 0) {
		    String fileName = "";
		    for (int i = 0; i < print.getPages().size(); i++) {
			fileName = getOutFileName();
			if (print.getPages().size() > 1) {
			    if ((i + 1) != print.getPages().size()) {
				fileName = fileName.concat(POINT).concat(
					new Integer(i + 1).toString());
			    } else {
				fileName = fileName.concat(POINT).concat(END);
			    }
			} else {
			    fileName = fileName.concat(POINT).concat(END);
			}
			fileNames.add(fileName);
		    }
		    setFileNames(fileNames);
		}
	    }

	    if ("html".equals(getOutFileFormat().toLowerCase())) {
		new JasperPrint().htmlExport(print, getOutFileName());
	    } else if ("txt".equals(getOutFileFormat().toLowerCase())) {
		new JasperPrint().plainTextExport(print, getOutFileName());
	    } else if ("rtf".equals(getOutFileFormat().toLowerCase())) {
		new JasperPrint().rtfExport(print, getOutFileName());
	    } else if ("xml".equals(getOutFileFormat().toLowerCase())) {
		new JasperPrint().xmlExport(print, getOutFileName());
	    } else if ("xls".equals(getOutFileFormat().toLowerCase())) {
		new JasperPrint().xlsExport(print, getOutFileName());
	    } else if ("print".equals(getOutFileFormat().toLowerCase())) {
		if (getNumberOfCopies() == 1) {
		    new JasperPrint().printExport(print, getPrinterName());
		} else {
		    new JasperPrint().printListExport(printList, getPrinterName());
		}
	    } else {
		new JasperPrint().pdfExport(print, getOutFileName());
	    }
	    if (isSendEmail()
		    && !"print".equals(getOutFileFormat().toLowerCase())) {
		new Emailer().sendMail(getEmails().split(EMAIL_DELIMITER),
			getSubject(), getBody(), getOutFileName());
	    }
	    if (isDoDelete()
		    && !"print".equals(getOutFileFormat().toLowerCase())) {
		new File(getOutFileName()).delete();
	    }
	    if (isDoDeleteXml()
		    && !"print".equals(getOutFileFormat().toLowerCase())) {
		new File(getInDataFile()).delete();
	    }
	} catch (JRException e) {
	    LOGGER.error("JRException: " + e);
	    System.exit(1);
	} catch (Exception e) {
	    LOGGER.error("Exception: " + e);
	    System.exit(1);
	}
    }

    /**
     * Validate and set command line arguments. Exit after printing usage if
     * anything is astray
     * 
     * @param args
     *                String[] args as featured in public static void main()
     */
    private void loadArgs(String[] args) {
	CommandLineParser parser = new PosixParser();
	try {
	    cmd = parser.parse(options, args);
	} catch (org.apache.commons.cli.ParseException e) {
	    LOGGER.error("Error parsing arguments: " + e);
	    System.exit(1);
	}

	// Check for mandatory args
	if (!cmd.hasOption(TEMPLATE_LOCATION_OPTION)) {
	    HelpFormatter formatter = new HelpFormatter();
	    formatter.printHelp("java -jar print.jar", options);
	    System.exit(1);
	}
	if (!cmd.hasOption(COMPILE_OPTION)) {
	    if ((!cmd.hasOption(FORMAT_OPTION) && !cmd
		    .hasOption(OUTPUT_FILE_OPTION))
		    || (cmd.hasOption(FORMAT_OPTION)
			    && !cmd.getOptionValue(FORMAT_OPTION).equals(
				    "print") && !cmd
			    .hasOption(OUTPUT_FILE_OPTION))) {
		HelpFormatter formatter = new HelpFormatter();
		formatter.printHelp(
			"Syntax is not correct.\njava -jar print.jar", options);
		System.exit(1);
	    }
	    if (cmd.hasOption(FORMAT_OPTION)
		    && "print".equals(cmd.getOptionValue(FORMAT_OPTION))
		    && !cmd.hasOption(PRINTER_OPTION)) {
		HelpFormatter formatter = new HelpFormatter();
		formatter.printHelp("Printer is not set.\njava -jar print.jar",
			options);
		System.exit(1);
	    }
	}

	// Look for optional args.
	if (cmd.hasOption(TEMPLATE_LOCATION_OPTION)) {
	    setReportFileName(cmd.getOptionValue(TEMPLATE_LOCATION_OPTION));
	}
	if (cmd.hasOption(OUTPUT_FILE_OPTION)) {
	    setOutFileName(cmd.getOptionValue(OUTPUT_FILE_OPTION));
	}
	if (cmd.hasOption(FORMAT_OPTION)) {
	    setOutFileFormat(cmd.getOptionValue(FORMAT_OPTION));
	}
	if (cmd.hasOption(DATA_LOCATION_OPTION)) {
	    setInDataFile(cmd.getOptionValue(DATA_LOCATION_OPTION));
	}
	if (cmd.hasOption(COMPILE_OPTION)) {
	    setDoCompile(true);
	}
	if (cmd.hasOption(PRINTER_OPTION)) {
	    setPrinterName(cmd.getOptionValue(PRINTER_OPTION));
	}
	if (cmd.hasOption(EMAIL_OPTION)) {
	    setEmails(cmd.getOptionValue(EMAIL_OPTION));
	}
	if (cmd.hasOption(SUBJECT_OPTION)) {
	    setSubject(cmd.getOptionValue(SUBJECT_OPTION));
	}
	if (cmd.hasOption(BODY_OPTION)) {
	    setBody(cmd.getOptionValue(BODY_OPTION));
	}
	if (cmd.hasOption(DELETE_AFTER_GENERATION)) {
	    setDoDelete(true);
	}
	if (cmd.hasOption(DELETE_DATA_AFTER_GENERATION)) {
	    setDoDeleteXml(true);
	}
	if (cmd.hasOption(SAVE_EACH_PAGE_ON_FILE)) {
	    setDoSaveSeparatly(true);
	}
	if (cmd.hasOption(NUMBER_OF_COPIES)) {
	    try {
		setNumberOfCopies(Integer.valueOf(
			cmd.getOptionValue(NUMBER_OF_COPIES)).intValue());
	    } catch (Exception e) {
	    }
	}

    }

    private boolean isSendEmail() {
	String emails = getEmails();
	return (emails != null && emails.length() > 0);
    }

    private String getFileExtension(String location) {
	String extension = "";
	if (location != null && location.length() > 0) {
	    File sourceFile = new File(location);
	    int dotPos = sourceFile.getAbsolutePath().lastIndexOf(".") + 1;
	    extension = sourceFile.getAbsolutePath().substring(dotPos);
	}
	return extension;
    }

    private void pdfExport(net.sf.jasperreports.engine.JasperPrint print,
	    String out) {
	try {
	    JRPdfExporter pdfExporter = new JRPdfExporter();
	    pdfExporter.setParameter(JRExporterParameter.JASPER_PRINT, print);
	    if (getFileNames() != null && getFileNames().size() > 0) {
		for (int i = 0; i < getFileNames().size(); i++) {
		    pdfExporter.setParameter(
			    JRExporterParameter.OUTPUT_FILE_NAME, fileNames
				    .get(i));
		    pdfExporter.setParameter(JRExporterParameter.PAGE_INDEX,
			    new Integer(i));
		    pdfExporter.exportReport();
		}
	    } else {
		pdfExporter.setParameter(JRExporterParameter.OUTPUT_FILE_NAME,
			out);
		pdfExporter.exportReport();
	    }
	} catch (JRException e) {
	    LOGGER.error("JRException: " + e);
	}
    }

    private void htmlExport(net.sf.jasperreports.engine.JasperPrint print,
	    String out) {
	try {
	    JRHtmlExporter htmlExporter = new JRHtmlExporter();
	    htmlExporter.setParameter(JRExporterParameter.JASPER_PRINT, print);
	    if (getFileNames() != null && getFileNames().size() > 0) {
		for (int i = 0; i < getFileNames().size(); i++) {
		    htmlExporter.setParameter(
			    JRExporterParameter.OUTPUT_FILE_NAME, fileNames
				    .get(i));
		    htmlExporter.setParameter(JRExporterParameter.PAGE_INDEX,
			    new Integer(i));
		    htmlExporter.exportReport();
		}
	    } else {
		htmlExporter.setParameter(JRExporterParameter.OUTPUT_FILE_NAME,
			out);
		htmlExporter.exportReport();
	    }

	} catch (JRException e) {
	    LOGGER.error("JRException: " + e);
	}
    }

    private void plainTextExport(net.sf.jasperreports.engine.JasperPrint print,
	    String out) {
	try {
	    JRTextExporter textExporter = new JRTextExporter();
	    textExporter.setParameter(JRExporterParameter.JASPER_PRINT, print);
	    textExporter.setParameter(JRTextExporterParameter.CHARACTER_WIDTH,
		    new Integer(10));
	    textExporter.setParameter(JRTextExporterParameter.CHARACTER_HEIGHT,
		    new Integer(10));
	    if (getFileNames() != null && getFileNames().size() > 0) {
		for (int i = 0; i < getFileNames().size(); i++) {
		    textExporter.setParameter(
			    JRExporterParameter.OUTPUT_FILE_NAME, fileNames
				    .get(i));
		    textExporter.setParameter(JRExporterParameter.PAGE_INDEX,
			    new Integer(i));
		    textExporter.exportReport();
		}
	    } else {
		textExporter.setParameter(JRExporterParameter.OUTPUT_FILE_NAME,
			out);
		textExporter.exportReport();
	    }
	} catch (JRException e) {
	    LOGGER.error("JRException: " + e);
	}
    }

    private void rtfExport(net.sf.jasperreports.engine.JasperPrint print,
	    String out) {
	try {
	    JRRtfExporter rtfExporter = new JRRtfExporter();
	    rtfExporter.setParameter(JRExporterParameter.JASPER_PRINT, print);
	    if (getFileNames() != null && getFileNames().size() > 0) {
		for (int i = 0; i < getFileNames().size(); i++) {
		    rtfExporter.setParameter(
			    JRExporterParameter.OUTPUT_FILE_NAME, fileNames
				    .get(i));
		    rtfExporter.setParameter(JRExporterParameter.PAGE_INDEX,
			    new Integer(i));
		    rtfExporter.exportReport();
		}
	    } else {
		rtfExporter.setParameter(JRExporterParameter.OUTPUT_FILE_NAME,
			out);
		rtfExporter.exportReport();
	    }
	} catch (JRException e) {
	    LOGGER.error("JRException: " + e);
	}
    }

    private void xlsExport(net.sf.jasperreports.engine.JasperPrint print,
	    String out) {
	try {
	    JRXlsExporter xlsExporter = new JRXlsExporter();
	    xlsExporter.setParameter(JRExporterParameter.JASPER_PRINT, print);
	    xlsExporter.setParameter(
		    JRXlsExporterParameter.IS_WHITE_PAGE_BACKGROUND,
		    Boolean.FALSE);
	    xlsExporter.setParameter(
		    JRXlsExporterParameter.IS_REMOVE_EMPTY_SPACE_BETWEEN_ROWS,
		    Boolean.TRUE);
	    if (getFileNames() != null && getFileNames().size() > 0) {
		for (int i = 0; i < getFileNames().size(); i++) {
		    xlsExporter.setParameter(
			    JRExporterParameter.OUTPUT_FILE_NAME, fileNames
				    .get(i));
		    xlsExporter.setParameter(JRExporterParameter.PAGE_INDEX,
			    new Integer(i));
		    xlsExporter.exportReport();
		}
	    } else {
		xlsExporter.setParameter(JRExporterParameter.OUTPUT_FILE_NAME,
			out);
		xlsExporter.exportReport();
	    }
	} catch (JRException e) {
	    LOGGER.error("JRException: " + e);
	}
    }

    private void xmlExport(net.sf.jasperreports.engine.JasperPrint print,
	    String out) {
	try {
	    JRXmlExporter xmlExporter = new JRXmlExporter();
	    xmlExporter.setParameter(JRExporterParameter.JASPER_PRINT, print);
	    if (getFileNames() != null && getFileNames().size() > 0) {
		for (int i = 0; i < getFileNames().size(); i++) {
		    xmlExporter.setParameter(
			    JRExporterParameter.OUTPUT_FILE_NAME, fileNames
				    .get(i));
		    xmlExporter.setParameter(JRExporterParameter.PAGE_INDEX,
			    new Integer(i));
		    xmlExporter.exportReport();
		}
	    } else {
		xmlExporter.setParameter(JRExporterParameter.OUTPUT_FILE_NAME,
			out);
		xmlExporter.exportReport();
	    }
	} catch (JRException e) {
	    LOGGER.error("JRException: " + e);
	}
    }

    private void printExport(net.sf.jasperreports.engine.JasperPrint print,
	    String printerLocation) {
	try {
	    PrinterName printerName = new PrinterName(printerLocation, null);
	    PrintServiceAttributeSet printServiceAttributeSet = new HashPrintServiceAttributeSet();
	    printServiceAttributeSet.add(printerName);
	    JRPrintServiceExporter exporter = new JRPrintServiceExporter();
	    exporter.setParameter(JRExporterParameter.JASPER_PRINT, print);
	    exporter
		    .setParameter(
			    JRPrintServiceExporterParameter.PRINT_SERVICE_ATTRIBUTE_SET,
			    printServiceAttributeSet);
	    exporter.setParameter(
		    JRPrintServiceExporterParameter.DISPLAY_PAGE_DIALOG,
		    Boolean.FALSE);
	    exporter.setParameter(
		    JRPrintServiceExporterParameter.DISPLAY_PRINT_DIALOG,
		    Boolean.FALSE);
	    PrintRequestAttributeSet aset = new HashPrintRequestAttributeSet();
	    aset.add(new Copies(1));
	    exporter
		    .setParameter(
			    JRPrintServiceExporterParameter.PRINT_REQUEST_ATTRIBUTE_SET,
			    aset);
	    exporter.exportReport();
	} catch (JRException e) {
	    LOGGER.error("JRException: " + e);
	}
    }

    private void printListExport(List list, String printerLocation) {
	for (Iterator iterator = list.iterator(); iterator.hasNext();) {
	    net.sf.jasperreports.engine.JasperPrint print = (net.sf.jasperreports.engine.JasperPrint) iterator.next();
	    printExport(print, printerLocation);
	}
    }
    
    private boolean isDoCompile() {
	return doCompile;
    }

    private void setDoCompile(boolean doCompile) {
	this.doCompile = doCompile;
    }

    private String getInDataFile() {
	return inDataFile;
    }

    private void setInDataFile(String inDataFile) {
	this.inDataFile = inDataFile;
    }

    private String getOutFileFormat() {
	return outFileFormat;
    }

    private void setOutFileFormat(String outFileFormat) {
	this.outFileFormat = outFileFormat;
    }

    private String getOutFileName() {
	return outFileName;
    }

    private void setOutFileName(String outFileName) {
	this.outFileName = outFileName;
    }

    private String getPrinterName() {
	return printerName;
    }

    private void setPrinterName(String printerName) {
	this.printerName = printerName;
    }

    private String getReportFileName() {
	return reportFileName;
    }

    private void setReportFileName(String reportFileName) {
	this.reportFileName = reportFileName;
    }

    public String getBody() {
	if (body == null) {
	    return "";
	}
	return body;
    }

    public void setBody(String body) {
	this.body = body;
    }

    public String getEmails() {
	return emails;
    }

    public void setEmails(String emails) {
	this.emails = emails;
    }

    public String getSubject() {
	if (subject == null) {
	    return "";
	}
	return subject;
    }

    public void setSubject(String subject) {
	this.subject = subject;
    }

    public boolean isDoDelete() {
	return doDelete;
    }

    public void setDoDelete(boolean doDelete) {
	this.doDelete = doDelete;
    }

    public boolean isDoDeleteXml() {
	return doDeleteXml;
    }

    public void setDoDeleteXml(boolean doDeleteXml) {
	this.doDeleteXml = doDeleteXml;
    }

    public int getNumberOfCopies() {
	if (numberOfCopies <= 0) {
	    return 1;
	}
	return numberOfCopies;
    }

    public void setNumberOfCopies(int numberOfCopies) {
	this.numberOfCopies = numberOfCopies;
    }

    public boolean isDoSaveSeparatly() {
	return doSaveSeparatly;
    }

    public void setDoSaveSeparatly(boolean doSaveSeparatly) {
	this.doSaveSeparatly = doSaveSeparatly;
    }

    public List getFileNames() {
	return fileNames;
    }

    public void setFileNames(List fileNames) {
	this.fileNames = fileNames;
    }

}
