package com.progress.print;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRImage;
import net.sf.jasperreports.engine.JRImageRenderer;
import net.sf.jasperreports.engine.JRRenderable;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.lowagie.text.pdf.Barcode128;
import com.lowagie.text.pdf.Barcode39;
import com.lowagie.text.pdf.BarcodeEAN;
import com.lowagie.text.pdf.BarcodeInter25;
import com.lowagie.text.pdf.BarcodePostnet;

public class XmlParser {
    private static final Logger LOGGER = Logger.getLogger(XmlParser.class);

    private String fileName;

    private Document doc;

    public void parse() {
	setDoc(parseFile(getFileName()));
    }

    public String getRootName() {
	return getDoc().getDocumentElement().getNodeName();
    }

    public HashMap getParameters() {
	Document doc = getDoc();
	Element docEle = doc.getDocumentElement();
	HashMap hm = new HashMap();
	NodeList nl = docEle.getElementsByTagName("parameter");
	if (nl != null && nl.getLength() > 0) {
	    for (int i = 0; i < nl.getLength(); i++) {
		Element el = (Element) nl.item(i);
		Parameter parameter = getParameter(el);
		if ("barcode".equals(parameter.getName().toLowerCase())) {
		    String rotate = getAttrValue(el, "rotate");
		    String type = getAttrValue(el, "type");
		    Image barcode;
		    if ("barcode128".equals(type.toLowerCase())) {
			Barcode128 code128 = new Barcode128();
			code128.setCode(parameter.getValue());
			barcode = code128.createAwtImage(Color.BLACK,
				Color.WHITE);
		    } else if ("barcode39".equals(type.toLowerCase())) {
			Barcode39 code39 = new Barcode39();
			code39.setCode(parameter.getValue());
			code39.setStartStopText(false);
			barcode = code39.createAwtImage(Color.BLACK,
				Color.WHITE);
		    } else if ("barcodeinter25".equals(type.toLowerCase())) {
			BarcodeInter25 code25 = new BarcodeInter25();
			code25.setGenerateChecksum(true);
			code25.setCode(parameter.getValue());
			barcode = code25.createAwtImage(Color.BLACK,
				Color.WHITE);
		    } else if ("barcodepostnet".equals(type.toLowerCase())) {
			BarcodePostnet codePost = new BarcodePostnet();
			codePost.setCode(parameter.getValue());
			barcode = codePost.createAwtImage(Color.BLACK,
				Color.WHITE);
		    } else {
			BarcodeEAN codeEAN = new BarcodeEAN();
			codeEAN.setCodeType(codeEAN.EAN13);
			codeEAN.setCode(parameter.getValue());
			barcode = codeEAN.createAwtImage(Color.BLACK,
				Color.WHITE);
		    }

		    BufferedImage destinationBI;
		    BufferedImage bufferedImageOut = new BufferedImage(barcode
			    .getWidth(null), barcode.getHeight(null),
			    BufferedImage.TYPE_INT_RGB);
		    Graphics g = bufferedImageOut.getGraphics();
		    g.drawImage(barcode, 0, 0, null);

		    if ("V".equals(rotate.toUpperCase())) {
			AffineTransform at = new AffineTransform();
			at.scale(1.0, 1.0);
			at.rotate(90.0 * Math.PI / 180.0, bufferedImageOut
				.getWidth() / 1.0,
				bufferedImageOut.getHeight() / 1.0);

			AffineTransform translationTransform;
			translationTransform = findTranslation(at,
				bufferedImageOut);
			at.preConcatenate(translationTransform);

			BufferedImageOp bio = new AffineTransformOp(at,
				AffineTransformOp.TYPE_BILINEAR);

			destinationBI = new BufferedImage(bufferedImageOut
				.getHeight(null), bufferedImageOut
				.getWidth(null), BufferedImage.TYPE_INT_RGB);
			bio.filter(bufferedImageOut, destinationBI);
		    } else {
			destinationBI = bufferedImageOut;
		    }

		    JRRenderable image = null;
		    try {
			image = JRImageRenderer.getInstance(destinationBI,
				JRImage.ON_ERROR_TYPE_ERROR);
			hm.put("barcode", image);
		    } catch (JRException e) {
			LOGGER.error(e);
		    }
		} else {
		    hm.put(parameter.getName(), parameter.getValue());
		}
	    }
	}
	return hm;
    }

    /*
     * find proper translations to keep rotated image correctly displayed
     */
    private AffineTransform findTranslation(AffineTransform at, BufferedImage bi) {
	Point2D p2din, p2dout;

	p2din = new Point2D.Double(0.0, 0.0);
	p2dout = at.transform(p2din, null);
	double ytrans = p2dout.getY();

	p2din = new Point2D.Double(0, bi.getHeight());
	p2dout = at.transform(p2din, null);
	double xtrans = p2dout.getX();

	AffineTransform tat = new AffineTransform();
	tat.translate(-xtrans, -ytrans);
	return tat;
    }

    /**
     * Parses XML file and returns XML document.
     * 
     * @param fileName XML file to parse
     * @return XML document or <B>null</B> if error occured
     */
    private Document parseFile(String fileName) {
	DocumentBuilder docBuilder;
	Document doc = null;
	DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory
		.newInstance();
	docBuilderFactory.setIgnoringElementContentWhitespace(true);
	try {
	    docBuilder = docBuilderFactory.newDocumentBuilder();
	} catch (ParserConfigurationException e) {
	    return null;
	}
	File sourceFile = new File(fileName);
	try {
	    doc = docBuilder.parse(sourceFile);
	} catch (SAXException e) {
	    LOGGER.error("SAXException: " + e);
	    return null;
	} catch (IOException e) {
	    LOGGER.error("IOException: " + e);
	}
	return doc;
    }

    private Parameter getParameter(Element empEl) {
	String name = getAttrValue(empEl, "name");
	String value = getTextValue(empEl);

	Parameter param = new Parameter();
	param.setName(name);
	param.setValue(value);
	return param;
    }

    private String getTextValue(Element ele) {
	String textVal = null;
	try {
	    textVal = ele.getFirstChild().getNodeValue();
	} catch (Exception e) {
	}
	if (textVal == null) {
	    return "";
	}
	return textVal;
    }

    private String getAttrValue(Element ele, String attrName) {
	String textVal = null;
	textVal = ele.getAttribute(attrName);
	return textVal;
    }

    public String getFileName() {
	return fileName;
    }

    public void setFileName(String fileName) {
	this.fileName = fileName;
    }

    public Document getDoc() {
	return doc;
    }

    public void setDoc(Document doc) {
	this.doc = doc;
    }

}
